/* jmixer.h
 * (c)2000 Denis Roio aka jaromil
 * based on the eMixer code by Dunstan B. Nesbit
 */

#ifndef __JMIXER_H
#define __JMIXER_H

#include <unistd.h>
#include <stdlib.h>

#include "libsync/synchronous.h"

extern "C" {
#include "shout.h"
#include "lame/lame.h"
#include "mpeg.h"
}

#define BUF_SIZE 2048
#define MAX_SAMPLE 32766
#define CLIP 30000
#define CLIP_A 7650756 // ((MAX_SAMPLE-CLIP)*(MAX_SAMPLE-CLIP))
#define CLIP_B -27234 // (MAX_SAMPLE-2*CLIP)
#define MAX_CHANNELS 8
#define MAX_PLAYLIST_ENTRY 150
#define MAX_PATH_SIZE 150
#define QUALITY 1 /* best=1 worse=9 */

struct livein {
  short inbuf[BUF_SIZE];
  int rate;
  int channels;
  bool on;
};

struct channel {
  short inbuf[BUF_SIZE];
  char *name;
  float volume;
  float speed;
  int rate;
  int channels;
  bool on;
  bool end;
  int stream[2];
  pid_t pid;
};

class Stream_mixer: public LTI::Object {

private:

  short audio_buffer[BUF_SIZE];
  int process_buffer[BUF_SIZE];
  short work1[BUF_SIZE*40], work2[BUF_SIZE];
  short inbuf[BUF_SIZE*40];

  int dsp;
  bool fullduplex;
  shout_conn_t ice_conn;
  lame_global_flags enc_flags;
  
  /* they're still hardcoded and initialized into the constructor */
  int sample_rate;
  int stereo;

  struct channel stream_channel[MAX_CHANNELS];
  struct livein live_channel;
  
  bool write_toicecast(short *sendbuf, long len);
  void close_soundcard();

  unsigned int get_audio(int ch);
  unsigned int get_livein();
  int spawn_stream(int ch);
  void ended_channel(int ch);

  void make_mono(short *stereo, short *mono, int num);
  void make_stereo(short *mono, short *stereo, int num);
  void sep_stereo(short *stereo, short *left, short *right, int num);
  void mix_stereo(short *left, short *right, short *stereo, int num);

  void int_compress_samples(short *in, short *out, int num, int grain);
  void int_stretch_samples(short *in, short *out, int num, int grain);

  void compress_samples(short *in, short *out,unsigned int num1,unsigned int num2);
  void stretch_samples(short *in, short *out,unsigned int num1,unsigned int num2);
  void change_time(short *in, short *out, unsigned int num_in, unsigned int num_out);
  void convert_audio(short *in,short *out,int ch1,int ch2,int n1,int n2);
  
public:
  Stream_mixer();

  void run();
  
  bool open_soundcard();
  bool open_icecast(char *ip, int port, char *pass, char *mountpoint, int bps);
  void play_chunks();
  void start_channel(int ch);
  void pause_channel(int ch);
  void stop_channel(int ch);
  void set_channel(int ch, int playlist_pos);
  void set_volume(int ch, float vol);
  void set_speed(int ch, float speed);
  void dump_status();
  void set_live(bool stat);

  /* this is for the libsync - registering synchronized method's ids */
  static const int id_set_channel = 1;
  static const int id_stop_channel = 2;
  static const int id_pause_channel = 3;
  static const int id_start_channel = 4;
  static const int id_set_volume = 5;
  static const int id_set_speed = 6;
  static const int id_dump_status = 7;
  static const int id_set_live = 8;

  bool dspout;
  bool icecastout;

  char playlist[MAX_PLAYLIST_ENTRY][MAX_PATH_SIZE];
  void set_playlist(char *pl);

  int *client;

  ~Stream_mixer();
};

#endif
